@@ -20,7 +20,9 @@ def authorize_token(key, secret, url, oauth_params):
         client = oauthclient.SimpleOAuthClient(SERVER, PORT, url, oauth_params['ACCESS_TOKEN_URL'], oauth_params['AUTHORIZATION_URL'])
         oauth_request = oauthlib.OAuthRequest.from_token_and_callback(token=fsRequestToken, http_url=client.authorization_url)
         try:
-                response = client.authorize_token(oauth_request)
+                (result, errstr, response) = client.authorize_token(oauth_request)
+                if not result:
+                        return (False, errstr, None, None)
                 connectionResult = True
         except socket.gaierror, e:
                 connectionResult = False
@@ -54,7 +56,9 @@ def fetch_request_token(url,params, oauth_params):
         oauth_request.sign_request(signature_method_plaintext, consumer, None)
         connectionResult = False
         try:
-                authResult = client.fetch_request_token(oauth_request)
+                (result, errstr, authResult) = client.fetch_request_token(oauth_request)
+                if not result:
+                        return (result, errstr, None)
                 connectionResult = True
         except socket.gaierror, e:
                 connectionResult = False
@@ -116,22 +120,44 @@ def api_parse_xml_response(xml):
 
         if parseSuccess == True:
                 errorNode = parsedXml.getElementsByTagName('error')
+                unAuthorizedNode = parsedXml.getElementsByTagName('unauthorized')
+                rateLimitedNode = parsedXml.getElementsByTagName('ratelimited')
                 if errorNode.length == 1:
                         infoNoticeString = errorNode[0].firstChild.data
-                else:				
-                        unAuthorizedNode = parsedXml.getElementsByTagName('unauthorized')
-                        if unAuthorizedNode.length == 1 and unAuthorizedNode[0].firstChild.data == 'TOKEN_EXPIRED':
+                elif unAuthorizedNode.length == 1:
+                        message = unAuthorizedNode[0].firstChild.data
+                        if message.startswith('TOKEN_EXPIRED'):
                                 infoNoticeString = 'Error: foursquare.com token expired. Please sign back in'
+                        elif message.startswith('TIMESTAMP_REFUSED'):
+                                infoNoticeString = 'Error: foursquare.com timestamp refused. Check your system clock?'
                         else:
-                                valid = True
+                                infoNoticeString = message
+                elif rateLimitedNode.length == 1:
+                        infoNoticeString = 'Error: you have exceeded the foursquare.com API ratelimit'
+                else:
+                        valid = True
+
+        if not valid:
+                print "api_parse_xml_response() error: " + infoNoticeString
 
         return (valid, infoNoticeString, parsedXml)
 
 def api_request_and_parse(url, parameters, oauth_params, method="GET"):
         print "foursquare.api_request_and_parse(%s, %s, %s, %s)" % (url, str(parameters), str(oauth_params), method)
-        (connectionResult, errstr, checkinsResultsXml) = api_request(url, parameters, oauth_params, method)
+        (connectionResult, errstr, resultXml) = api_request(url, parameters, oauth_params, method)
         if not connectionResult:
+                print "api_request() failed: " + errstr
                 return (False, errstr, None)
-        return api_parse_xml_response(checkinsResultsXml)
+
+        print url
+        print resultXml
+
+        (result, errstr, parsed) = api_parse_xml_response(resultXml)
+
+        if not result:
+                print "api_request_and_parse() failed: " + errstr
+
+        return (result, errstr, parsed)
+
 
 
